# DataprooferTest

[index.js:30-85](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L30-L85 "Source code on GitHub")

Creates an individual test instance.

**Parameters**

-   `options` **Object** contains testing functions
    -   `options.methdology`  **MUST BE INCLUDED TO WORK.** Begins testing and defaults to empty result object

**Examples**

```javascript
var DataprooferTest = require('DataprooferTest');
var myTest = new DataprooferTest({
  methodology: function(rows, columnHeads, input) {
     var testState;
     if (rows.length > 0) {
       testState = "passed";
     } else {
       testState = "failed";
     }
     // RESULTS MUST INCLUDE THESE FIVE KEY VALUES
     return {
        testState: testState, // the result of the test
        highlightedCells: [...] // array of cell objects to highlight
     }
  }
});

// {
//   testState: "failed",
//   highlightedCells: [...]
// };
```

## conclusion

[index.js:265-289](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L265-L289 "Source code on GitHub")

Get a test's conclusion —
next steps someone should take if a test does not pass.

**Parameters**

-   `input`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.conclusion();

// "This spreadsheet has more than one row."
```

Returns **String** containing a conclusion if a test does not pass

## conclusion

[index.js:265-289](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L265-L289 "Source code on GitHub")

Set a test's description with a string dependent on the test result

**Parameters**

-   `containing` **Function** a conclusion if a test or does not pass
-   `input`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.conclusion(function(result) {
  if(result.testState === "passed") {
    return "Passed. You may use this dataset"
  } else {
    return "You may not use this dataset"
  }
});
```

Returns **DataprooferTest** 

## conclusion

[index.js:265-289](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L265-L289 "Source code on GitHub")

Set a test's description with a string independent of the test result

**Parameters**

-   `containing` **String** a conclusion if a test or does not pass
-   `input`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.conclusion("This spreadsheet has more than one row.");
```

Returns **DataprooferTest** 

## description

[index.js:138-152](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L138-L152 "Source code on GitHub")

Get a test's description

**Parameters**

-   `descriptionString`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.description();

// "Counts the rows of a spreadsheet"
```

Returns **String** containing a description of the test

## description

[index.js:138-152](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L138-L152 "Source code on GitHub")

Set a test's description

**Parameters**

-   `providing` **String** a description of the test
-   `descriptionString`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.description("Counts the rows of a spreadsheet");
```

Returns **DataprooferTest** 

## doesPass

[index.js:225-231](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L225-L231 "Source code on GitHub")

Runs a user-specified test and returns the result as a string

**Parameters**

-   `rows`  an array of row objects; column heads are keys, cells are values. (optional, default `[]`)
-   `columnHeads`  an array of column head names (optional, default `[]`)
-   `input`  

**Examples**

```javascript
var myTest = new DataprooferTest({...});
myTest.doesPass(rows, columnHeads, input);

// "failed"
```

Returns **Object** default result object describing the test

## methodology

[index.js:171-185](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L171-L185 "Source code on GitHub")

Set a test's proofing function, aka its methodology

**Parameters**

-   `that` **Function** takes a function and returns an object. See {DataprooferTest}
-   `methodologyFunction`  

**Examples**

```javascript
var myTest = new DataprooferTest({...});
myTest.methodology(function);
```

Returns **DataprooferTest** 

## methodology

[index.js:171-185](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L171-L185 "Source code on GitHub")

Get a test's proofing function, aka its methodology

**Parameters**

-   `methodologyFunction`  

**Examples**

```javascript
var myTest = new DataprooferTest({...});
myTest.methodology();

// function(rows, columnHeads, input) {...}
```

Returns **Function** describing the test's current functions

## name

[index.js:105-119](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L105-L119 "Source code on GitHub")

Set a test's name

**Parameters**

-   `providing` **String** a more explicative, full name for the test
-   `nameString`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.name("My test");
```

Returns **DataprooferTest** 

## name

[index.js:105-119](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L105-L119 "Source code on GitHub")

Get a test's full name

**Parameters**

-   `nameString`  

**Examples**

```javascript
var myTest = newDataprooferTest({...});
myTest.name();

// "My test"
```

Returns **String** containing the name of the test

## proof

[index.js:206-211](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/index.js#L206-L211 "Source code on GitHub")

Runs a user-specified test. If no test is specified, a default "results" object is returned

**Parameters**

-   `rows`  an array of row objects; column heads are keys, cells are values. (optional, default `[]`)
-   `columnHeads`  an array of column head names (optional, default `[]`)
-   `input`  an array objects representing user input (optional, default `[]`)

**Examples**

```javascript
var myTest = new DataprooferTest({...});
myTest.proof(rows, columnHeads, input);

// {
//   testState: "failed", # default boolean
//   name: "", # default empty string
//   description: "", # default empty string
//   summary: "", # default empty string
//   highlightedCells: [] # default empty array
// };
```

Returns **Object** default result object describing the test

# util

[util.js:5-5](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/util.js#L5-L5 "Source code on GitHub")

A set of comonly used utility functions for us inside tests

# isEmpty

[util.js:48-53](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/util.js#L48-L53 "Source code on GitHub")

Check if a cell's value is empty

**Parameters**

-   `value`  

Returns **boolean** 

# isNumeric

[util.js:20-27](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/util.js#L20-L27 "Source code on GitHub")

Check if a value is numeric (a cell's value could be a number in string form)

**Parameters**

-   `value`  

Returns **boolean** 

# isString

[util.js:34-41](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/util.js#L34-L41 "Source code on GitHub")

Check if a value is numeric (a cell's value could be a number in string form)

**Parameters**

-   `value`  

Returns **boolean** 

# percent

[util.js:74-77](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/util.js#L74-L77 "Source code on GitHub")

return a string representing the percentage of a fraction

**Parameters**

-   `fraction`  

Returns **String** 

# stripNumeric

[util.js:60-67](https://github.com/dataproofer/dataproofertest-js/blob/3158ce4614b87a9745b3d20d52c3fe544c70c6b2/util.js#L60-L67 "Source code on GitHub")

Strip a cell of the following characters: "$", ",", "%"

**Parameters**

-   `value`  

Returns **String** 
